<?php

namespace HCM\FPA\Services;

use Doctrine\ORM\EntityManager;
use HCM\Util\DateUtil;
use HCM\Util\ManipulacaoExcel;
use HCM\Util\FPA;
use HCM\Util\NovoCodigo;
use HCM\Util\HCMParameters;
use HCM\Util\Messages;
use HCM\Util\Repositories;
use HCM\Geral\Services\FileService;
use HCM\Geral\Services\UploadFileService;
use HCM\FPA\Services\ExportacaoExcelService;
use HCM\FPA\Services\ItemLoteLancService;
use HCM\FPA\Calculos\Apolo\MigracaoFilaApolo;
use HCM\FPA\Entity\FpacalculofolhaEntity;
use Zeedhi\Framework\HTTP\Kernel;
use Zeedhi\Framework\Session\Session;

define('APP_PATH', dirname(__FILE__) . "/");

class MigracaoFichaFinanceiraService {
    
    public function __construct(EntityManager $entityManager, HCMParameters $hcmParameters, Kernel $httpKernel, ExportacaoExcelService $exportacaoExcelService, ItemLoteLancService $itemLoteLancService) {
        $this->entityManager = $entityManager;
        $this->hcmParameters = $hcmParameters;
        $this->httpKernel = $httpKernel;
        $this->mensagens = [];
        $this->exportacaoExcelService = $exportacaoExcelService;
        $this->itemLoteLancService = $itemLoteLancService;
        $this->fpaeventom = Repositories::FPA_EVENTOM;
        $this->fpaeventoh = Repositories::FPA_EVENTOH;
    }
    
    public function baixarPlanilhaTemplateEventosDePara($row) {
        $nrorg = $this->hcmParameters->getNrorgTrabalho();
        $nrorgpadrao = $this->hcmParameters->getNrorgPadrao();
        $dtmescompetencia = $this->hcmParameters->getDtCompetencia();
        
        $events = $this->entityManager->getRepository(Repositories::FPA_EVENTOM)->retornaEventos($nrorg, $nrorgpadrao, $dtmescompetencia);
        
        foreach ($events as $evento) {
            $eventos[] = array("dtinivigencia"  => $evento->getDtinivigencia()->format('d/m/Y'), 
                               "idtipoevento"   => $evento->getIdtipoevento(),
                               "dtmescompetenc" => '01/01/2000',
                               "nmeventoh"      => $evento->historico->getNmeventoh(),
                               "nreventom"      => $evento->getNreventom());
        }
        
        $cabecalho = array(
            "Data início",
            "Tipo Evento",
            "Competência",
            "Descrição",
            "Nº Evento Pebbian",
            "Nº Evento Cliente");
            
        return $this->exportacaoExcelService->gravarTemplate($eventos, $nrorg, $cabecalho, "Eventos De/Para");
        
    }
    
    public function baixarPlanilhaTemplateLoteFicha($row) {
        $nrorg = $this->hcmParameters->getNrorgTrabalho();
        $nrorgpadrao = $this->hcmParameters->getNrorgPadrao();
        $dtmescompetencia = $this->hcmParameters->getDtCompetencia();
        
        $exemplo[] = ['tpmoviment'    => 1,
                      'nrvinculom'    => 1,
                      'nreventom'     => 1,
                      'dtocorrencia'  => '05/01/2020',
                      'vreventofolha' => '1000',
                      'vrreffolha'    => '30',
                      'dscalculo'     => 'Item de exemplo'];

        $cabecalho = array(
            "Tp movimento",
            "Nr vínculo",
            "Nr evento",
            "Dt ocorrência",
            "Valor",
            "Valor da Referência",
            "Descrição do cálculo");
            
        return $this->exportacaoExcelService->gravarTemplate($exemplo, $nrorg, $cabecalho, "Lote Ficha Financeira");
        
    }
    
    public function importarPlanilhaEventoDePara($row){
        try{
            $this->entityManager->beginTransaction();
            
            $this->mensagens = [];
            $migrandepara = Repositories::MIGRAMDEPARA;
            $nrorg = $this->hcmParameters->getNrorgTrabalho();
            $nrorgpadrao = $this->hcmParameters->getNrorgPadrao();
            $dtCompetencia = DateUtil::getDataDeString($this->hcmParameters->getDtCompetencia(), DateUtil::FORMATO_BRASILEIRO, true);
            $salvo = false;
            
            $infos = $this->preparaImportacao($row);
            
            if(is_array($infos)){
                foreach($infos as $key => $dados) {
                    //atualiza o valor da linha pra mostrar erros, se necessário
                    $linha = $key+2;
                    
                    $existe = $this->entityManager->getRepository(Repositories::MIGRAMDEPARA)->findOneBy(array('nrorg' => $nrorg, 'nrde' => $dados['NRDE']));
                    // $existe = $this->verificaExistenciaCtbl($nrorg, $nrorgpadrao, $dados['EVENTO'], $dtCompetencia, $dados['CONTDEBITO'], $dados['HISTPDRDEB'], $dados['CONTCREDIT'], $dados['HISTPDRCRE']);
                    
                    if($existe){
                        $this->mensagens[] = "Evento ".$dados['NRDE']." já foi cadastrado.";
                    }else{
                        $evento = $this->entityManager->getRepository(Repositories::FPA_EVENTOM)->retornaEvento($dados['NRPARA'], $nrorg, $nrorgpadrao, $dtCompetencia);
                        
                        if(empty($evento)){
                            if($dados['TPEVENTO'] == 'DEMONSTRATIVO'){
                                $ideventodemonst = 'S';
                            }else{
                                $ideventodemonst = 'N';
                            }
                            $evento = $this->novoEvento($nrorg, DateUtil::getDataDeString($dados['DTINICIO'], DateUtil::FORMATO_BRASILEIRO, true), $dados['TPEVENTO'], DateUtil::getDataDeString($dados['DTMESCOMPETENC'], DateUtil::FORMATO_BRASILEIRO, true), 
                                                        $dados['NMEVENTOH'], 'COMPETENCIA', $ideventodemonst, 'N', 'N', 'N', 'N', 'N', 'N', 'N', 'N', 'CALCULO_PRINCIPAL', 'N', 'N', 'N');
                            $this->mensagens[] = "Foi adicionado o evento ".$evento[0]->getNreventom()." do HCM. Verifique suas informações.";
                            $dados['NRPARA'] = $evento[0]->getNreventom();  
                        }
                        
                        $eventoDePara = $this->novoMigramdepara($nrorg, $dados['NRDE'], $dados['NRPARA'], 'IMPORTACAO', 'EVENTO', $dados['NRDE'], $dados['NRPARA'], $dados['NMEVENTOH'], null);
                        $this->mensagens[] = "Foi importado registro do evento ".$dados['NRDE']." do cliente.";
                        $salvo = true;
                    }
                    unset($existe);
                    unset($evento);
                }
            }
            
            $this->entityManager->commit();
            return array('success'  => $salvo,
                         'messages' => $this->mensagens);
        } catch (\Exception $e) {
            $this->entityManager->rollback();
            throw $e;
        }    
    }
    
    public function importarPlanilhaLoteFicha($row){
        set_time_limit(0);
        error_reporting(E_ERROR | E_PARSE);
        try{
            $this->entityManager->beginTransaction();
            
            $this->mensagens = [];
            $fpa_lotelancevento = Repositories::FPA_LOTELANCEVENTO;
            $nrorg = $this->hcmParameters->getNrorgTrabalho();
            $nrorgpadrao = $this->hcmParameters->getNrorgPadrao();
            $nrtipomoviment = 1;
            $dtCompetencia = DateUtil::getDataDeString($this->hcmParameters->getDtCompetencia(), DateUtil::FORMATO_BRASILEIRO, true);
            $salvo = false;
            
            
            $dslotelancevento = $row['DSPATH'][0]['name'];
            $dtlotelancevento = $this->hcmParameters->getDataAtual();
            
            $infos = $this->preparaImportacao($row, 'loteFicha');
            
            $qtdeitenstotal = count($infos);
            
            $lotelancevento = $this->novoLotelancevento($nrorg, $nrtipomoviment, $dslotelancevento, DateUtil::getDataDeString($dtlotelancevento, DateUtil::FORMATO_BRASILEIRO, true), $dtCompetencia, 'DIGITACAO', null, 'S', 'MIGRACAO', $dtCompetencia, $qtdeitenstotal);
            
            if(!empty($infos) && is_array($infos) && empty($this->mensagens)){
                
                $dataSet = array('ROW'              => $infos,
                                 'NRLOTELANCEVENTO' => $lotelancevento->getNrlotelancevento());
                
                $migracaoFilaApolo = new MigracaoFilaApolo($this->entityManager, $this->hcmParameters, $dataSet);
                $mensagem = $migracaoFilaApolo->importacaoItensLoteFicha();
                
                $this->entityManager->commit();
                return array('success'  => true,
                             'messages' => $mensagem);
            }else{
                return array('success'  => false,
                             'messages' => $this->mensagens);
            }
            
        } catch (\Exception $e) {
            $this->entityManager->rollback();
            throw $e;
        }
    }
    
    public function exportarLoteFicha($row){
        set_time_limit(0);
        error_reporting(E_ERROR | E_PARSE);
        try{
            $this->entityManager->beginTransaction();
            
            $this->mensagens = [];
            $fpa_itecalcfolha = Repositories::FPA_ITECALCFOLHA;
            $nrorg = $this->hcmParameters->getNrorgTrabalho();
            $nrorgpadrao = $this->hcmParameters->getNrorgPadrao();
            $nrtipomoviment = 1;
            $dtCompetencia = DateUtil::getDataDeString($this->hcmParameters->getDtCompetencia(), DateUtil::FORMATO_BRASILEIRO, true);
            $salvo = false;
            $fpacalculofolhaentity = new FpacalculofolhaEntity($this->entityManager, $this->hcmParameters);
            
            $calculoFolha = $fpacalculofolhaentity->retornaCalculoFolha($nrorg, $row['NRTIPOMOVIMENT'], DateUtil::getDataDeString($row['DTOCORRENCIA'], DateUtil::FORMATO_BRASILEIRO, true), 1, 1);
            $itenslanclote = $this->entityManager->getRepository(Repositories::FPA_ITEMLOTELANC)->findBy(array('nrorg' => $nrorg, 'nrlotelancevento' => $row['NRLOTELANCEVENTO']));
            
            if(!empty($itenslanclote)){
                // $this->entityManager->beginTransaction();
                
                $dataSet = array('ROW'              => $itenslanclote,
                                 'NRCALCULOFOLHA'   => $calculoFolha[0]->getNrcalculofolha(),
                                 'NRLOTELANCEVENTO' => $row['NRLOTELANCEVENTO']);
                    
                $migracaoFilaApolo = new MigracaoFilaApolo ($this->entityManager, $this->hcmParameters, $dataSet);
                $mensagem = $migracaoFilaApolo->exportacaoItensLoteFicha();
                
                // $this->entityManager->commit();
                return array('success'  => true,
                             'messages' => $mensagem);
            }else{
                return array('success'  => false,
                             'messages' => ['Não foi encontrado itens para exportar para a folha.']);
            }
            
        } catch (\Exception $e) {
            $this->entityManager->rollback();
            throw $e;
        }
    }
    
    public function cancelarLoteFichaFinanceira($row){
        try{
            $nrorg = $this->hcmParameters->getNrorgTrabalho();
            $dtmescompetenc = $this->hcmParameters->getDtCompetencia();
            $temLiquido = false;
            $calculofolha = null;
            $mensagens = array();
            $dsorigmsg = 'LOTEEVENTO'.$row['NRLOTELANCEVENTO'];
            
            $this->entityManager->beginTransaction();
            
            $fpacalculofolha = $this->entityManager->getRepository(Repositories::FPA_CALCULOFOLHA)->retornaCalculoFolhaLoteFicha($nrorg, $dtmescompetenc, $row['NRTIPOMOVIMENT'], $row['NRLOTELANCEVENTO']);
            
            $calculofolha = !empty($fpacalculofolha) ? $fpacalculofolha[0] : null;
            
            $itenslanclote = $this->entityManager->getRepository(Repositories::FPA_ITEMLOTELANC)->retornaItensLoteLancFicha($nrorg, $row['NRLOTELANCEVENTO'], $dtmescompetenc);
            
            if ($calculofolha) {
    
                foreach ($itenslanclote as $itemlanclote) {
                    
                    $itensliquido = $this->itemLoteLancService->getLiquido($itemlanclote->getNrvinculom(), $calculofolha->getNrcalculofolha(), $nrorg);
    
                    if ($itensliquido) {
                        $temLiquido = true;
                        array_push($mensagens, "Já existe líquido para o vinculo ".$itemlanclote->getNrvinculom().".");
                    }
                }
                
                if ($temLiquido){
                    
                    array_push($mensagens, "Lote não foi cancelado.");
                }else{    
                    
                    // $this->entityManager->getRepository(Repositories::FPA_ITECALCFOLHA)->findBy(array('nrorg' => $nrorg, 'nrcalculofolha' => $calculofolha->getNrcalculofolha(), 'nrlotelancevento' => $row['NRLOTELANCEVENTO']));
                    
                    // foreach ($itenscalculo as $itemcalc){
                    //     $itemcalc->delete();
                    // }
                    // $this->entityManager->flush();
                    
                    $query = $this->entityManager->getConnection()->prepare("
                        DELETE FROM FPA_ITECALCFOLHA WHERE NRORG = :NRORG AND NRCALCULOFOLHA = :NRCALCULOFOLHA AND NRLOTELANCEVENTO = :NRLOTELANCEVENTO");
            
                    $query->execute([
                        "NRORG"            => $nrorg,
                        "NRCALCULOFOLHA"   => $calculofolha->getNrcalculofolha(),
                        "NRLOTELANCEVENTO" => $row['NRLOTELANCEVENTO']]);
                    
                    
                    /*Atualiza o status do lote de eventos para pendente*/
                    $lote = $this->entityManager->getRepository(Repositories::FPA_LOTELANCEVENTO)->findOneBy(array('nrorg' => $nrorg, 'nrlotelancevento' => $row['NRLOTELANCEVENTO']));
                    $lote->setIdstatuslotelanc('CANCELADO');
                    
                    $this->entityManager->persist($lote);
                    $this->entityManager->flush();
                    
    
                    $mensagens = true;
                }
                
            }else{
                array_push($mensagens, "Cálculo referente ao itens não foi encontrado.");
                array_push($mensagens, "Lote não foi cancelado.");
            }
            
            $this->entityManager->commit();
            return $mensagens; 
        }catch (\Exception $e) {
            $this->entityManager->rollback();
            throw $e;
        }
    } 
    
    public function preparaImportacao ($row, $tipo = 'eventoDePara') {
        $arquivo = $row['DSPATH'][0];
        $nrorg = $this->hcmParameters->getNrorgTrabalho();
        $file = $this->httpKernel->factoryFromFileData($arquivo, FPA::getTempDirectory()."/");
        
        if (!$file->isFile()) {
            $this->mensagens[] = "Falha ao importar o arquivo.";
        }
        $caminhoNovo = DIRECTORY_SEPARATOR . $file->getPathname();
        $this->validaExtensao($caminhoNovo, $arquivo['name']);
        if($tipo == 'eventoDePara'){
            return $this->preparaInformacaoEventoDePara($caminhoNovo, $arquivo['name']);
        }else if($tipo == 'loteFicha'){
            return $this->preparaInformacaoLoteFicha($caminhoNovo, $arquivo['name']);
        }
    }
    
    private function validaExtensao($caminho, $arquivo) {
        $nome = preg_split("/(\\|\/|\.)/", $caminho);
        $extencao = strtoupper(end($nome));
        switch ($extencao) {
            case('XLS'):{
                return 1;
            }
            case('XLSX'):{
                return 2;
            }
            default:{
                $this->mensagens[] = "Extensão Inválida. (" . $arquivo . ")";
            }
        }
    }
    
    public function preparaInformacaoEventoDePara($caminho, $nome) {
        try {
            $nrorg = $this->hcmParameters->getNrorgTrabalho();
            $file = \PHPExcel_IOFactory::load($caminho);
            $file->setActiveSheetIndex(0);
            
            $eventos = $this->exportacaoExcelService->preparaInfoEvenDePara($file);
            
            if(is_array($eventos)){
                $eventos = $this->validaFormatacaoEvento($eventos, $nome);
            }else{
                $this->mensagens[] = 'Não foi possível importar! Todas as linhas devem tem Nr evento cliente.';
            }
            return $eventos;
            
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function preparaInformacaoLoteFicha($caminho, $nome) {
        try {
            $nrorg = $this->hcmParameters->getNrorgTrabalho();
            $file = \PHPExcel_IOFactory::load($caminho);
            $file->setActiveSheetIndex(0);
            
            $itens = $this->exportacaoExcelService->preparaInfoLoteFicha($file);
            
            if(is_array($itens)){
                $eventos = $this->validaFormatacaoItens($itens, $nome);
            }
            
            return $itens;
            
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    
    private function validaFormatacaoEvento($eventos, $nomeArquivo){
        if (!empty($eventos)) {
            foreach ($eventos as $key => &$evento) {
                if (intval($evento['NRDE']) == 0) {
                    $this->mensagens[] = "Evento inválido inserido na linha " . ++$key;
                } else {
                    $eventosValidos[] = $evento;
                }
            }
            
            return $eventosValidos;
        } else {
            $this->mensagens[] = "Não há eventos.";
            return false;
        }
    }
    
    private function validaFormatacaoItens($itens, $nomeArquivo){
        $eventosValidos = [];
        if (!empty($itens)) {
            foreach ($itens as $key => &$item) {
                if (!empty($item['NRVINCULOM'])) {
                    $eventosValidos[] = $item;
                }
            }
            
            return $eventosValidos;
        } else {
            return false;
        }
    }
    
    public function preparaMensagens($mensagens){
        $arrayRetorno = array();
        foreach($mensagens as $mensagem){
            $arrayRetorno[] = ['msg' => $mensagem];
        }
        return $arrayRetorno;
    }
    
    public function novoEvento($nrorg, $dtinivigencia, $idtipoevento, $dtmescompetenc, $nmeventoh, $iddtocorrencia, $ideventodemonst, $idfaltas13resc, $idimpressevento, $idmediaevenferi, $idrepdecterc, $idrepdsr, $idrepferias, $idreprecisao, 
                               $idretrocedeprov, $idretrocedevent, $idzeraevenfecha, $idcompsalfer, $idcompsalprov, $dtfimvigencia = null, $cdintegraevento = null, $idativo = 'S', $dsobservacao = null, $idfaltasferias = null, $idgeravrzero = null, 
                               $idmediavalref = null, $idprocdifmovant = null, $idprocdifrescis = null, $idtipoarredonda = null, $idtipoocorreven = null, $idunidrefereven = null, $nrdigarredonda = null, $nreventoestorno = null, $nreventoneg = null, 
                               $nreventorescis = null, $nreventotransf = null, $nrhomologrubrica = null, $nrincidcp = null, $nrincidfgts = null, $nrincidirrf = null, $nrincidsind = null, $nrnaturubr = null, $nrordimpreseven = null, $nrpriorestorno = null, 
                               $qtlimirefereven = null, $qtminocorconsec = null, $qtminocormedfer = null, $vrlimitearred = null, $vrlimiteevento = null, $vrpelimitevento = null, $vrpercentualevento = null, $vrpisoevento = null, $vrtetoevento = null){
        
        $eventom = $this->eventoMestre($nrorg, $dtinivigencia, $idtipoevento, $dtfimvigencia);
        
        $eventoh = $this->eventoHistorico($nrorg, $eventom->getNreventom(), $dtinivigencia, $dtmescompetenc, $nmeventoh, $iddtocorrencia, $ideventodemonst, $idfaltas13resc, $idimpressevento, $idmediaevenferi, $idrepdecterc, $idrepdsr, $idrepferias, $idreprecisao,
                                          $idretrocedeprov, $idretrocedevent, $idzeraevenfecha, $idcompsalfer, $idcompsalprov, $cdintegraevento, $idativo, $dsobservacao, $idfaltasferias, $idgeravrzero, $idmediavalref, $idprocdifmovant, $idprocdifrescis, 
                                          $idtipoarredonda, $idtipoocorreven, $idunidrefereven, $nrdigarredonda, $nreventoestorno, $nreventoneg, $nreventorescis, $nreventotransf, $nrhomologrubrica, $nrincidcp, $nrincidfgts, $nrincidirrf, $nrincidsind, $nrnaturubr, 
                                          $nrordimpreseven, $nrpriorestorno, $qtlimirefereven, $qtminocorconsec, $qtminocormedfer, $vrlimitearred, $vrlimiteevento, $vrpelimitevento, $vrpercentualevento, $vrpisoevento, $vrtetoevento);
    
        return array($eventom, $eventoh);
    }
    
    public function eventoMestre($nrorg, $dtinivigencia, $idtipoevento, $dtfimvigencia = null){
        
        $eventom = new $this->fpaeventom;
    
        $eventom->setNrorg($nrorg);
        $eventom->setDtinivigencia($dtinivigencia);
        $eventom->setDtfimvigencia($dtfimvigencia);
        $eventom->setIdtipoevento($idtipoevento);


        $eventom->setDtinclusao(DateUtil::getDataAtual());
        $eventom->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
        $eventom->setCdoperinclusao($this->hcmParameters->getCdoperador());
        $eventom->setDtultatu(DateUtil::getDataAtual());
        $eventom->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
        $eventom->setCdoperultatu($this->hcmParameters->getCdoperador());

        $eventom->setNreventom(\HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgPadrao(), 'FPA_EVENTOM', 1, 12, true));
        
        $this->entityManager->persist($eventom);
        $this->entityManager->flush();
        
        return $eventom;
    }
    
    public function eventoHistorico($nrorg, $nreventom, $dtinivigencia, $dtmescompetenc, $nmeventoh, $iddtocorrencia, $ideventodemonst, $idfaltas13resc, $idimpressevento, $idmediaevenferi, $idrepdecterc, $idrepdsr, $idrepferias, $idreprecisao, $idretrocedeprov, 
                                    $idretrocedevent, $idzeraevenfecha, $idcompsalfer, $idcompsalprov, $cdintegraevento = null, $idativo = 'S', $dsobservacao = null, $idfaltasferias = null, $idgeravrzero = null, $idmediavalref = null, $idprocdifmovant = null, 
                                    $idprocdifrescis = null, $idtipoarredonda = null, $idtipoocorreven = null, $idunidrefereven = null, $nrdigarredonda = null, $nreventoestorno = null, $nreventoneg = null, $nreventorescis = null, $nreventotransf = null, 
                                    $nrhomologrubrica = null, $nrincidcp = null, $nrincidfgts = null, $nrincidirrf = null, $nrincidsind = null, $nrnaturubr = null, $nrordimpreseven = null, $nrpriorestorno = null, $qtlimirefereven = null, $qtminocorconsec = null, 
                                    $qtminocormedfer = null, $vrlimitearred = null, $vrlimiteevento = null, $vrpelimitevento = null, $vrpercentualevento = null, $vrpisoevento = null, $vrtetoevento = null){

        $eventoh = new $this->fpaeventoh;

        $eventoh->setNrorg($nrorg);
        $eventoh->setNreventom($nreventom);
        $eventoh->setCdintegraevento($cdintegraevento);
        
        $eventoh->setDtmescompetenc($dtmescompetenc);
        // $eventoh->setDtinivigencia($dtinivigencia);
        $eventoh->setIdativo($idativo);
        $eventoh->setIddtocorrencia($iddtocorrencia);
        $eventoh->setIdeventodemonst($ideventodemonst);
        $eventoh->setDsobservacao($dsobservacao);
        $eventoh->setIdfaltasferias($idfaltasferias);
        $eventoh->setIdfaltas13resc($idfaltas13resc);
        $eventoh->setIdgeravrzero($idgeravrzero);
        $eventoh->setIdimpressevento($idimpressevento);
        $eventoh->setIdmediaevenferi($idmediaevenferi);
        $eventoh->setIdmediavalref($idmediavalref);
        $eventoh->setIdprocdifmovant($idprocdifmovant);
        $eventoh->setIdprocdifrescis($idprocdifrescis);
        $eventoh->setIdrepdecterc($idrepdecterc);
        $eventoh->setIdrepdsr($idrepdsr);
        $eventoh->setIdrepferias($idrepferias);
        $eventoh->setIdreprecisao($idreprecisao);
        $eventoh->setIdretrocedeprov($idretrocedeprov);
        $eventoh->setIdretrocedevent($idretrocedevent);
        $eventoh->setIdtipoarredonda($idtipoarredonda);
        $eventoh->setIdtipoocorreven($idtipoocorreven);
        $eventoh->setIdunidrefereven($idunidrefereven);
        $eventoh->setIdzeraevenfecha($idzeraevenfecha);
        $eventoh->setNrdigarredonda($nrdigarredonda);
        $eventoh->setNreventoestorno($nreventoestorno);
        $eventoh->setNreventoneg($nreventoneg);
        $eventoh->setNreventorescis($nreventorescis);
        $eventoh->setNreventotransf($nreventotransf);
        $eventoh->setNrhomologrubrica($nrhomologrubrica);
        $eventoh->setNrincidcp($nrincidcp);
        $eventoh->setNrincidfgts($nrincidfgts);
        $eventoh->setNrincidirrf($nrincidirrf);
        $eventoh->setNrincidsind($nrincidsind);
        $eventoh->setNrnaturubr($nrnaturubr);
        $eventoh->setNrordimpreseven($nrordimpreseven);
        $eventoh->setNrpriorestorno($nrpriorestorno);
        $eventoh->setQtlimirefereven($qtlimirefereven);
        $eventoh->setQtminocorconsec($qtminocorconsec);
        $eventoh->setQtminocormedfer($qtminocormedfer);
        $eventoh->setVrlimitearred($vrlimitearred);
        $eventoh->setVrlimiteevento($vrlimiteevento);
        $eventoh->setVrpelimitevento($vrpelimitevento);
        $eventoh->setVrpercentualevento($vrpercentualevento);
        $eventoh->setVrpisoevento($vrpisoevento);
        $eventoh->setVrtetoevento($vrtetoevento);
        $eventoh->setIdcompsalfer($idcompsalfer);
        $eventoh->setIdcompsalprov($idcompsalprov);
    
        $eventoh->setDtinclusao(DateUtil::getDataAtual());
        $eventoh->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
        $eventoh->setCdoperinclusao($this->hcmParameters->getCdoperador());
        $eventoh->setDtultatu(DateUtil::getDataAtual());
        $eventoh->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
        $eventoh->setCdoperultatu($this->hcmParameters->getCdoperador());

        $eventoh->setNreventoh(\HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgPadrao(), 'FPA_EVENTOH', 1, 12, true));
        if(empty($nmeventoh)){
            $nmeventoh = 'Evento migrado - '.$eventoh->getNreventoh();
        }
        $eventoh->setNmeventoh($nmeventoh);
        
        $this->entityManager->persist($eventoh);
        $this->entityManager->flush();
        
        return $eventoh;
    
    }
    
    public function novoMigramdepara($nrorg, $nrde, $nrpara, $idmigram = null, $nmtabeladbf = null, $idcoddbf = null, $idcodfpa = null, $dscampodbf = null, $idmigramemp = null){
        
        $repository = Repositories::MIGRAMDEPARA;
        $migramdepara = new $repository;
        
        $migramdepara->setIdmigram($idmigram);
        $migramdepara->setNmtabeladbf($nmtabeladbf);
        $migramdepara->setIdcoddbf($idcoddbf);
        $migramdepara->setIdcodfpa($idcodfpa);
        $migramdepara->setDscampodbf($dscampodbf);
        $migramdepara->setNrorg($nrorg);
        $migramdepara->setIdmigramemp($idmigramemp);
        $migramdepara->setNrpara($nrpara);
        $migramdepara->setNrde($nrde);
        $migramdepara->setDtinclusao(DateUtil::getDataAtual());
        $migramdepara->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
        $migramdepara->setCdoperinclusao($this->hcmParameters->getCdoperador());
        
        $migramdepara->setNrmigramdepara(\HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'MIGRAMDEPARA', 1, 12, true));
        
        $this->entityManager->persist($migramdepara);
        $this->entityManager->flush();
    }
    
    public function novoLotelancevento($nrorg, $nrtipomoviment, $dslotelancevento, $dtlotelancevento, $dtcompetloteeve, $idstatuslotelanc, $dsobslotelanc = null, $idativo = 'S', $idoriglotelanc = null, $dtocorrencia= null, $qtdeitenstotal = null){
        
        $repository = Repositories::FPA_LOTELANCEVENTO;
        $lotelancevento = new $repository;
        
        $lotelancevento->setDslotelancevento($dslotelancevento);
        $lotelancevento->setDtlotelancevento($dtlotelancevento);
        $lotelancevento->setDtcompetloteeve($dtcompetloteeve);
        $lotelancevento->setIdstatuslotelanc($idstatuslotelanc);
        $lotelancevento->setDsobslotelanc($dsobslotelanc);
        $lotelancevento->setIdativo($idativo);
        $lotelancevento->setNrorg($nrorg);
        $lotelancevento->setNrtipomoviment($nrtipomoviment);
        $lotelancevento->setIdoriglotelanc($idoriglotelanc);
        $lotelancevento->setDtocorrencia($dtocorrencia);
        $lotelancevento->setQtdeitenstotal($qtdeitenstotal);
        $lotelancevento->setDtinclusao(DateUtil::getDataAtual());
        $lotelancevento->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
        $lotelancevento->setCdoperinclusao($this->hcmParameters->getCdoperador());
        
        $lotelancevento->setNrlotelancevento(\HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'FPA_LOTELANCEVENTO', 1, 12, true));
        
        $this->entityManager->persist($lotelancevento);
        $this->entityManager->flush();
        return $lotelancevento; 
    
    }
}