<?php

namespace HCM\Geral\Controllers;

use HCM\Geral\Services\AquisicaoProdutoRuralService;
use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use HCM\Util\HCMParameters;

class AquisicaoProdutoRuralController extends Crud {

    protected $dataSourceName = 'eso_aquisicaoprodutorural';
    protected $dataSourceNameGet = 'getAquisicaoProdutoRural';
    
    protected $parameterBagColumns = array('P_NRORG', 'P_NRORG_PADRAO', 'P_DTMESCOMPETENC');
    
    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters, AquisicaoProdutoRuralService $aquisicaoProdutoRuralService){
        $this->parameterBag = $parameterBag;
        $this->hcmParameters = $hcmParameters;
        $this->aquisicaoProdutoRuralService = $aquisicaoProdutoRuralService;
        parent::__construct($dataSourceManager);
    }
    
    public function importarPlanilhaAquisicaoProdutoRural(DTO\Request\DataSet $request, DTO\Response $response) {
        set_time_limit(0);
        
        try{
            $row = $request->getDataSet()->getRows();
            $row = $row[0];
            
            $messages = $this->aquisicaoProdutoRuralService->importarPlanilhaAquisicaoProdutoRural($row);
            if(empty($messages)){
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message("Operação concluida com sucesso.", \Zeedhi\Framework\DTO\Response\Message::TYPE_MESSAGE));
            } else {
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message("Operação concluida. Existem erros na importação", \Zeedhi\Framework\DTO\Response\Message::TYPE_WARNING));
                $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('msg', $messages));
            }
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function getAquisicaoProdutoRural(DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceNameGet);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }
    
    public function setAquisicaoProdutoRural(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            foreach ($request->getDataSet()->getRows() as &$row) {
                if( array_key_exists('NRAQUIPRODRURAL', $row) ){
                    if( $row['NRAQUIPRODRURAL'] === '' ){
                        $row['NRAQUIPRODRURAL'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgPadrao(), 'ESO_AQUIPRODRURAL', 1, 12, true); 
                    }
                } else {
                    $row['NRAQUIPRODRURAL'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgPadrao(), 'ESO_AQUIPRODRURAL', 1, 12, true); 
                }
            }
            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function deleteAquisicaoProdutoRural(DTO\Request\DataSet $request, DTO\Response $response){
        try{
            parent::delete($request, $response);
        } catch(\Exception $e){
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }    
    }
    
    protected function processConditionValue($condition) {
        return $condition['value'];     
    }
    
    protected function populateParameterBag(FilterCriteria $filterCriteria) {
        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if( in_array($columnName, $this->parameterBagColumns) ){
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );
        return $newFilterCriteria;
    }
    
} 