<?php
namespace HCM\PTO\Repository;

use Doctrine\DBAL\Connection;
use Doctrine\ORM\EntityManager;
use HCM\Util\Repositories;

class HorarioRepository {

    protected  $connection;
    public function __construct(Connection $connection, EntityManager $entityManager) {
        $this->connection    = $connection;
        $this->entityManager = $entityManager;
    }

    /**
     * @return array
     */
    public function removeSaidaHorario($dataset) {
        $query = $this->entityManager->getConnection()->prepare("
               UPDATE GPE_INTERVALOSHORARIO
                  SET HRPROJETARINTERVALO = null, DSINTERVALOPROJETADO = null
                WHERE NRORG              = :NRORG
                  AND NRHORDIAM          = :NRHORDIAM
                  AND NRINTERVALOHORARIO = :NRINTERVALOHORARIO
                  AND NRSEQINTERVHORARIO = :NRSEQINTERVHORARIO");

        $return = $query->execute(
            array(
                "NRORG"               => $dataset["NRORG"],
                "NRHORDIAM"           => $dataset["NRHORDIAM"],
                "NRSEQINTERVHORARIO"  => $dataset["NRSEQINTERVHORARIO"],
                "NRINTERVALOHORARIO"  => $dataset["NRINTERVALOHORARIO"]
            )
        );
        return $return;
    }
    
    /**
     *  Rotina que verifica se os intervalos de horário não incluem marcações 'DA' ou 'FA' que remetem à dias anteriores
     * 
     *  @param      $nrorg
     *  @param      $nrintervalohorario
     * 
     *  @return     Array   array('error' => $errorMesage, 'response' => boolean)
     */
    public function horarioIncluiDiaAnterior($nrorg, $nrhordiam) {
        $status = [
            'error' => null,
            'response' => false,
        ];
        
        $intervalosHorario = $this->entityManager->getRepository(Repositories::GPE_INTERVALOSHORARIO)->findBy(array('nrorg' => $nrorg, 'nrhordiam' => $nrhordiam));;
        if (empty($intervalosHorario)) {
            $status['error'] = 'Intervalos de horário não foram encontrados';
            return $status;
        }
        
        $horarioVira = false;
        foreach($intervalosHorario as $intervaloHorario) {
            if ($intervaloHorario->getIddiaocorreentrada() == 'DA' || $intervaloHorario->getIddiaocorresaida() == 'DA') {
                $horarioVira = true;
            }
        }
        
        $status['response'] = $horarioVira;
        return $status;
    }

} 

